/*************************************************************************
 * Copyright (c) 2004 Nick T Tate
 * 
 * This file is part of Tearless SDK.
 * 
 * Tearless SDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Tearless SDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Tearless SDK; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ************************************************************************/
/*
 * This is a Tearless SDK sample program.
 * 
 * It shows how to use part of the API, but isn't a very good
 * example since it mixes several ways of doing things.
 * 
 * When you run it you hopefully will see a row of familiar icons.
 * With the left and right keys you should be able to change which
 * icon is highlighted. This is the first part which (barely) shows
 * how forms work.
 * 
 * The second part is initiated with the display button. This button
 * toggles between the normal background and the wide background,
 * if your device has such a mode. The wide background is, as you
 * might have guessed, much larger than the visible screen. By using
 * the left and right buttons again you can pan around in this wide
 * background (they still change the icons though). In addition, by
 * using zoom in and zoom out you may move the background up and down.
 * There are no limits as to how far you can move so you might fall
 * off the edge of the draw background. This is intentional so you can
 * explore the memory of your device.
 * 
 * Exit the program with the menu key.
 * 
 * Have fun!
 * 
 * Author: Nick T Tate 2004-07-19
 * 
 * $Id: pan.cpp 15 2004-10-21 14:53:25Z nicktt $
 */

#include <stdio.h>
#include "tlapi.h"

#define OVERLAY_WIDTH 640
#define NUM_BUTTONS 7

static BOOL BAD_BATT = FALSE;
static ushort BG_START_X = 0;
static ushort BG_START_Y = 0;
static short BG_DELTA_X = 0;
static short BG_DELTA_Y = 0;
static BYTE BG_TYPE = 0;

// Some more or less random images
static ushort IMG_NUMS[] = { 56, 57, 58, 59, 60, 61, 62, 0 };

// A nice palette
static Palette PALETTE = {
	0x3F00,	0xD830,	0xC8D7,	0xD830,
	0xC83F,	0xC03F,	0xC83F,	0x0000,
	0xDBC2,	0xCCC1,	0xCCCC,	0xFE00,
	0xE5A9,	0xE800,	0xFE00,	0xC000
};

static Label LABEL;
static char LABEL_STR[80];

BYTE TLCALL msgFlagsHandler(Msg eventCode) {
	/* Always update battery icon, always beep.
	 * (Still, silent or release messages never beep.)
	 */
	return 1;
};

/* Helper function to update the visible offset of the background.
 */
void updateBgPos(BOOL show) {
	if ((BG_DELTA_X == 0) && (BG_DELTA_Y == 0)) return;

	BG_START_X += BG_DELTA_X;
	BG_START_Y += BG_DELTA_Y;
	GFX.setVisibleOffset(0, BG_START_X, BG_START_Y);
	if (show) {
		GFX.showPage(0);
	}
}

ERR TLCALL halEventHandler(HalEvent* event, WBOOL* keepRunning) {
	long keyStates;
	WORD err = ERR_NONE;
	Msg msg = MSG_FLAG_SILENT;
	BOOL flipPage = FALSE;
	
	switch (event->type) {
		case keyStateChange:
			keyStates = ((KeyStateEvent*)event)->keyStates;

			// Left and right keys: Pan and translate to message
			if ((keyStates & FLAG_RIGHT) != 0) {
				BG_DELTA_X = -16;
				updateBgPos(FALSE);
				msg = MSG_KEY_RIGHT;
			} else if ((keyStates & FLAG_LEFT) != 0) {
				BG_DELTA_X = 16;
				updateBgPos(FALSE);
				msg = MSG_KEY_LEFT;
			} else {
				BG_DELTA_X = 0;
			};

			// Zoom in and out: Pan up and down
			if ((keyStates & FLAG_ZOOM_IN) != 0) {
				BG_DELTA_Y = 16;
				updateBgPos(TRUE);
			} else if ((keyStates & FLAG_ZOOM_OUT) != 0) {
				BG_DELTA_Y = -16;
				updateBgPos(TRUE);
			} else {
				BG_DELTA_Y = 0;
			};

			// Menu and disp keys: Exit or change bg width
			if ((keyStates & FLAG_MENU) != 0) {
				/* Since FORM_SETTINGS doesn't handle MSG_KEY_MENU,
				 * send cleanup message instead ...
				 */
				msg = (Msg) (MSG_CLEANUP | MSG_FLAG_SILENT);
			} else if ((keyStates & FLAG_DISP) != 0) {
				// Toggle bg type.
				BG_TYPE = 1 - BG_TYPE;
				BG_START_X = 0;
				BG_START_Y = 0;
				GFX.selectBitmapWidth(0, BG_TYPE);
				GFX.setVisibleOffset(0, BG_START_X, BG_START_Y);
				// Send this so we may get a beep (and a screen update)
				msg = MSG_KEY_DISP;
			};
			
			if (msg != MSG_FLAG_SILENT) {
				flipPage = TRUE;
			};
			break;
		case runLevelChange:
			/* Display this so you know. Normally you would leave
			 * if this happens (set keepRunning to FALSE).
			 */
			LABEL.x = 44;
			GFX.fgFillRect(44, 274, 88, 48, 0);
			sprintf(LABEL_STR, "RL:%d", ((RunLevelEvent*)event)->newRunLevel);
			LABEL.draw();
			flipPage = TRUE;
			break;
		case weakBattChange:
			/* Display this so you know you got one. The battery low icon
			 * should show up soon, but firmware might differ.
			 */
			LABEL.x = 512;
			GFX.fgFillRect(512, 274, 88, 48, 0);
			sprintf(LABEL_STR, "BT:%d", ((WeakBattEvent*)event)->minTimeLeft & 0xF);
			LABEL.draw();
			flipPage = TRUE;
			// Force low battery icon updates.
			BAD_BATT = TRUE;
			break;
		default:
			break;
	}

	if (BAD_BATT || flipPage) {
		err = GUI.sendFormMsg(msg, keepRunning);
		GFX.showPage(0);
	};

	return err;
};

ERR main(void) {
	short i;
	WORD err = ERR_NONE;
	WBOOL keepRunning = TRUE;
	Form *mainForm, *oldForm;

	// Important!
	loadTearlessLibs();

	LABEL.fgColor = 14;
	LABEL.drawMode = DRAW_NO_BG;
	LABEL.text = LABEL_STR;
	LABEL.y = 274;

	// Detect if we have a weak battery.
	if ((SYS.getBattTimeLeftEtc() & 0xF) != 6) {
		BAD_BATT = TRUE;
	}

	// Allocate form
	mainForm = GUI.allocForm(FORM_SETTINGS, NUM_BUTTONS);
	// Set form palette
	mainForm->palette = &PALETTE;
	// Assure we update the batt
	mainForm->msgFlagsHandler = &msgFlagsHandler;

	// Initialize selected button (depends on form type if needed).
	mainForm->buttons[0].widget.status = STATUS_SELECTED;

	// Init buttons
	for (i = 0; i < NUM_BUTTONS; i++) {
		Button* button = &mainForm->buttons[i];
		button->setPos(-1, 8);
		// FORM_SETTINGS relies on this so we must initialize it.
		button->msgHandler = NULL;
		button->setData(&IMG_NUMS[i]);
	}

	// Save old form if any
	GUI.getActiveForm(&oldForm);
	// Set our as the active
	mainForm->activate();

	// Just to be sure no one interferes with our INIT_MSG.
	GUI.setSingleMsgHandler(NULL);

	/* Init the dialog. This clears the page. (Then, most form types
	 * set the palette and draw themselves. FORM_SETTINGS doesn't,
	 * it relies on the MsgHandler for the selected button.)
	 */
	err = GUI.sendFormMsg(MSG_INIT, &keepRunning);
	// Draw manually
	mainForm->draw();
	// Show our form.
	GFX.showPage(0);

	// Main event loop	
	while((err == 0) && (keepRunning == 1)) {
		if ((BG_DELTA_X != 0) || (BG_DELTA_Y != 0)) {
			updateBgPos(!BAD_BATT);
		};

		if (BAD_BATT) {
			// Update battery low icon if needed.
			err = GUI.sendFormMsg(MSG_FLAG_SILENT, &keepRunning);
			if (err != ERR_NONE) break;
		};		

		err = SYS.processHalEvents((HalEventHandler)&halEventHandler, &keepRunning);
	}

	if (err != ERR_NONE) {
		/* Show error code before leaving, with a delay effect
		 * so you won't miss it. Note that the error may be
		 * that the battery is running out (ERR_BATT_DEAD), so
		 * you might want to check for that if you do something
		 * like this in your own code ...
		 */
		LABEL.setPos(0, 200);
		sprintf(LABEL_STR, "Error code:%04X", err);
		LABEL.draw();
		GFX.fgFillRect(0, 250, OVERLAY_WIDTH, 24, 2);
		GFX.showPage(0);
		for (i = OVERLAY_WIDTH-4; i >= 0; i -= 4) {
			GFX.fgFillRect(i, 250, 4, 24, 0);
			GFX.showPage(0);
		}
	}

	// Set bg bitmap to narrow to avoid confusion. (Try without it!)
	GFX.selectBitmapWidth(0, 0);
	
	// Restore default palette
	GFX.setFgPalette(NULL);
	// Clear overlay.
	GFX.fgFillPage(0);
	// Show the cleaned page
	GFX.showPage(0);

	// Restore old form
	oldForm->activate();
	// Free memory
	mainForm->free();
	
	// Keep our errors to ourselves for now ...
	return ERR_NONE;
};

