/*************************************************************************
 * Copyright (c) 2004 Nick T Tate
 * 
 * This file is part of Tearless SDK.
 * 
 * Tearless SDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Tearless SDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Tearless SDK; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ************************************************************************/
/*
 * Header file for Tearless Graphics library (Lib 7).
 *
 * Author: Nick T Tate 2003-09-07
 * 
 * $Id: tlgfx.h 12 2004-10-21 11:51:11Z nicktt $
 */

#ifndef TEARLESS_GFX_HEADER
#define TEARLESS_GFX_HEADER

#include "tlcommon.h"

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack (push, 1);

/*
 * Enums
 */

typedef enum VideoMode {
	VIDEO_LCD = 0,
	VIDEO_NTSC = 1,
	VIDEO_PAL = 2
} VideoMode;

typedef enum DrawMode {
	DRAW_NO_BG = 0,
	DRAW_BLANK_1 = 1,
	DRAW_BG_RECT = 2,
	DRAW_BITMAPPED_BG = 3,
	DRAW_BLANK_4 = 4,
	DRAW_HIGH_FG_RECT = 5
} DrawMode;

/*
 * Types
 */

typedef struct Label {
	ushort x, y;
	char __far * text;
	WORD drawMode;	// DrawMode
	COLOR fgColor;
	COLOR bgColor;

#ifdef TEARLESS_USE_CLASSES
	void draw(void);
	void setPos(ushort x, ushort y) { this->x = x; this->y = y; };
#endif
} Label;

typedef struct Icon {
	ushort x, y;
	ushort width, height;
	WORD drawMode;	// DrawMode
	DWORD imageHandle;
	WORD reserved_E;
	WORD reserved_10;
	COLOR fgColor;
	COLOR bgColor;

#ifdef TEARLESS_USE_CLASSES
	void draw(void);
	void setPos(ushort x, ushort y) { this->x = x; this->y = y; };
	void setSize(ushort width, ushort height) {
		this->width = width;
		this->height = height;
	};
#endif
} Icon;

typedef struct Rect {
	ushort x1, y1;
	ushort x2, y2;
	BYTE pixelsForZero;
	WORD reserved_9;
	COLOR color;

#ifdef TEARLESS_USE_CLASSES
	void fill(void);
	void setBBox(ushort x1, ushort y1, ushort x2, ushort y2) {
		this->x1 = x1;
		this->y1 = y1;
		this->x2 = x2;
		this->y2 = y2;
	};
#endif
} Rect;

typedef struct BorderRect {
	ushort x, y;
	ushort width, height;
	BYTE borderThickness;
	WORD reserved_9;
	WBOOL fillCenter;
	COLOR borderColor;
	COLOR centerColor;

#ifdef TEARLESS_USE_CLASSES
	void draw(void);
	void setPos(ushort x, ushort y) { this->x = x; this->y = y; };
	void setSize(ushort width, ushort height) {
		this->width = width;
		this->height = height;
	};
#endif
} BorderRect;

// Graphics lib
struct TearlessGfxLib {
	ERR (TLCALL *initLib)(BOOL manualBuffering, BYTE videoMode);
	void (TLCALL *freeVideoSlots)(void);

	/*
	 * TLS100: Only works on the background (layer 0) with indices 0 and 1.
	 * A 1 means to use the wider bitmap that can fit a zoomed in photo
	 * so panning is easier. This is likely to change on newer equipment.
	 */
	void (TLCALL *selectBitmapWidth)(BYTE layer, BYTE sizeIndex);

	void (TLCALL *selectDrawLayerAndPage)(BYTE layer, BYTE pageNum);
	void (TLCALL *setVisibility)(BYTE layer, BOOL visible);
	void (TLCALL *setVisibleOffset)(BYTE layer, ushort xOffset, ushort yOffset);

	/*
	 * If colors are NULL, the default palette is used.
	 */
	void (TLCALL *setFgPalette)(Palette* colors);
	ERR (TLCALL *setVideoMode)(VideoMode videoMode);

	/* Normally, this function is called with how = 0. Then:
	 * 
	 * For every layer, if anything has been drawn on the current
	 * page, the display HW is set to show the current page.
	 * In automatic buffering mode (normal) this page is copied
	 * to the next page which then becomes the current.
	 * 
	 * Exception: If the fg buffering is bypassed, the bypass
	 * is cleared and the old current page restored. No page
	 * is copied and the HW display address is not changed.
	 * 
	 * If how == 2, a completely black screen is shown.
	 * 
	 * In any case, also waits until at least 150 ticks after
	 * a video mode change. (May wait more.)
	 */
	ERR (TLCALL *showPage)(BYTE how);
	ERR (TLCALL *setVideoCombo)(BYTE mode);
	void (TLCALL *bypassFgBuffering)(BOOL bypass);

	/*
	 * Whether the video mode should change when the video cable
	 * is connected or disconnected.
	 */
	void (TLCALL *enableAutoSwitchToTV)(BOOL enable);
	VideoMode (TLCALL *getVideoMode)(void);
	void (TLCALL *fgFillPage)(COLOR color);
	void (TLCALL *fgFillPageFast)(COLOR color);
	void (TLCALL *bgFillTopLeftScreen)(COLOR color);
	void (TLCALL *fgFillRect)(ushort x, ushort y, ushort width, ushort height, COLOR color);

	/*
	 * NOTE: The coordinates are adjusted according to video mode. Due to
	 * the YUV format, X-coordinates and widths must be evenly divisible by 4.
	 * TLS100: Only Y-compensation, LCD -> +12, NTSC -> +12, PAL -> +4
	 */	
	void (TLCALL *bgFillRect)(ushort x, ushort y, ushort width, ushort height, COLOR color);

	/*
	 * Both 128 kB banks are used. This creates a CPU-visible area
	 * of 256 kB including at least 128 kB following the given address.
	 * A segmented pointer to where the address has been mapped is returned.
	 * 
	 * TLS100: This pointer is always in the range 4000:0000-5000:FFFF,
	 * where the segment is either 0x4000 or 0x5000 depending on bit 17 and
	 * the low 16 bits are the same as the given address. It can thus be
	 * advisable to adjust an address of say 5000:FFFF to 5FFF:000F.
	 */
	PTR (TLCALL *setTempMemBanks)(DWORD linearAddress);
	void (TLCALL *restoreMemBanks)(void);
	void (TLCALL *fgCopyAbsRect)(ushort srcX, ushort srcY, ushort destX, ushort destY, ushort width, ushort height);

	/*
	 * NOTE: The coordinates are adjusted according to video mode. Due to
	 * the YUV format, X-coordinates and widths must be evenly divisible by 4.
	 * TLS100: Only Y-compensation, LCD -> +12, NTSC -> +12, PAL -> +4
	 */	
	void (TLCALL *bgCopyRect)(ushort srcX, ushort srcY, ushort destX, ushort destY, ushort width, ushort height);
	void (TLCALL *fgCopyFirstPage)(ushort destAbsY);
	void (TLCALL *bgCopyCurrentPage)(ushort destAbsY);
	void (TLCALL *fgCopyFirstPageAlt)(ushort destAbsY);
	ERR (TLCALL *setAsciiFont)(DWORD fgBitmapLinPtr, DWORD bgBitmapLinPtr, ushort width, ushort height);
	ERR (TLCALL *setWideFont)(DWORD fgBitmapLinPtr, DWORD bgBitmapLinPtr, ushort width, ushort height);
	void (TLCALL *getFontDimensions)(Dimension* asciiDim, Dimension* wideDim);
	void (TLCALL *fillRectFromStruct)(Rect* rect);
	void (TLCALL *drawBorderRect)(BorderRect* rect);

	/*
	 * TLS100: Aware of the selected layer, but bg drawing is disabled.
	 */
	void (TLCALL *drawLabel)(Label* label);

	/*
	 * TLS100: Draws on the selected layer, but always as if it was
	 * foreground, resulting in garbage on bg.
	 */
	void (TLCALL *drawIcon)(Icon* icon);
	ERR (TLCALL *drawPhoto80)(char* fileName, ubyte zoom, ushort x, ushort y, ulong offset, ulong bytes, ushort angle);
	ERR (TLCALL *drawPhoto84)(char* fileName, ushort x, ushort y, ulong offset, ulong bytes, ushort angle);
	void (TLCALL *getLastPhotoScaledSize)(ushort* width, ushort* height);
	ERR (TLCALL *readFileToLinearMem)(char* fileName, DWORD linearAddress);
	ERR (TLCALL *func90)();
};

#define GFX (*((struct TearlessGfxLib*)(TEARLESS_LIBS[7])))

#ifdef TEARLESS_USE_CLASSES
/*
 * Inline wrappers for instance methods defined in library.
 */

	// Label
	inline void Label::draw(void) { GFX.drawLabel(this); };

	// Icon
	inline void Icon::draw(void) { GFX.drawIcon(this); };

	// Rect
	inline void Rect::fill(void) { GFX.fillRectFromStruct(this); };

	// BorderRect
	inline void BorderRect::draw(void) { GFX.drawBorderRect(this); };

#endif

#pragma pack (pop);
#ifdef __cplusplus
}
#endif

#endif
